TidyPlates = {}
local pairs = pairs
local select = select
local regionPosition = { "threatGlow", "healthBorder", "castBorder", "castNostop", 
					"spellIcon", "highlightTexture", "nameText", "levelText",
					"dangerSkull", "raidIcon", "eliteIcon" }
local regionContent = { threatGlow = "image", healthBorder = "image", castBorder = "image", 
					castNostop = "image", spellIcon = "image", highlightTexture = "image", 
					nameText = "text", levelText = "text", dangerSkull = "image", 
					raidIcon = "image", eliteIcon = "image"	}
----------------------------------------------
local numChildren = -1
local plateTheme = TidyPlatesThemeTemplate
----------------------------------------------
local PlateHandler = CreateFrame("Frame")
-- >>> ------------------------------------------------------------------------------------------------- >>> LoadStyle
local function LoadStyle(templateStyle, customStyle)
	local checkedStyle = {} 
	local propertylist = {}
	local category, property, defaultvalue, customvalue
	
	if type(customStyle) == "table" then
		for category, propertylist in pairs(templateStyle) do 
		  checkedStyle[category] = {}
		  for property, defaultvalue in pairs(propertylist) do
			checkedStyle[category][property] = defaultvalue
			if type(customStyle[category]) == "table" then customvalue = customStyle[category][property] else customvalue = nil end
			if type(customvalue) == type(defaultvalue) then checkedStyle[category][property] = customvalue end
		  end
		end
		return checkedStyle
	else return nil end
end
-- >>> ------------------------------------------------------------------------------------------------- >>> LoadMultiStyle
local function LoadMultiStyle(templateStyle, styleList)
	local style, styleData = {}
	local count = 0
	local checkedStyleList = {}
	local tempStyle = {}
	
	if type(styleList) == "table" then
		for style, styleData in pairs(styleList) do 
			tempStyle = LoadStyle(templateStyle, styleData)
			if tempStyle then 
				count = count + 1
				checkedStyleList[style] = tempStyle
			end
		end
		if count > 0 then return checkedStyleList else return nil end
	else return nil end
end
-- >>> ------------------------------------------------------------------------------------------------- >>> LoadMultiStyle
local function LoadTheme(template, theme)
	local checkedTheme = {}
	local multiStyle = false

	if type(theme) == "table" then
		if theme.SetStyle and type(theme.SetStyle) == "function" then
			checkedTheme = LoadMultiStyle(template, theme)
			if checkedTheme then multiStyle = true end
		else checkedTheme = LoadStyle(template, theme) end
		
		if checkedTheme then
			if type(theme.SetScale) == "function" then checkedTheme.SetScale = theme.SetScale end
			if type(theme.SetSpecialText) == "function" then checkedTheme.SetSpecialText = theme.SetSpecialText end
			if type(theme.SetSpecialText2) == "function" then checkedTheme.SetSpecialText2 = theme.SetSpecialText2 end
			if type(theme.SetSpecialArt) == "function" then checkedTheme.SetSpecialArt = theme.SetSpecialArt end
			if type(theme.SetAlpha) == "function" then checkedTheme.SetAlpha = theme.SetAlpha end
			if type(theme.SetStyle) == "function" then checkedTheme.SetStyle = theme.SetStyle end
			checkedTheme.multiStyle = multiStyle
			return checkedTheme
		end
	else return template end
end
-- >>> ------------------------------------------------------------------------------------------------- >>> 
local function shouldDisplayThreat(unit)
	if InCombatLockdown() and unit.reaction ~= "FRIENDLY" and IsThreatWarningEnabled() then return true 
	else return false end
end
-- >>> ------------------------------------------------------------------------------------------------- >>>
local function threatByColor( object , name)
	local redCan, greenCan, blueCan, alphaCan = object:GetVertexColor()
	local shown
	if not object:IsShown() then return "LOW" end
	if greenCan > .7 then return "MEDIUM" end
	if redCan > .7 then return "HIGH" end
end
-- >>> ------------------------------------------------------------------------------------------------- >>>
local function reactionByColor(object)																											
	local redCan, greenCan, blueCan, alphaCan = object:GetStatusBarColor()
	if redCan < .01 and blueCan < .01 and greenCan > .99 then return "FRIENDLY", "NPC" 
	elseif redCan < .01 and blueCan > .99 and greenCan < .01 then return "FRIENDLY", "PLAYER"
	elseif redCan > .99 and blueCan < .01 and greenCan > .99 then return "NEUTRAL", "NPC"
	else return "HOSTILE", "UNKNOWN" end
end
-- >>> ------------------------------------------------------------------------------------------------- >>>
local function SetFrameBySize(frame, width, height, anchor, offX, offY)																		
	frame:ClearAllPoints()
	frame:SetWidth(width)
	frame:SetHeight(height)
	frame:SetPoint(anchor, offX, offY)
end
-- >>> ------------------------------------------------------------------------------------------------- >>>		
local function SetFrameByStyle(frame, style, anchorTo, alpha)																						
	frame:ClearAllPoints()
	frame:SetWidth(style.width)
	frame:SetHeight(style.height)
	frame:SetPoint(style.anchor, anchorTo, style.anchor, style.x, style.y)
	frame:SetAlpha(alpha)
end
-- >>> ------------------------------------------------------------------------------------------------- >>>
local function ApplyBarSettings(statBar, statbarStyle, extended, alpha)																			
	if statbarStyle.texture ~= statBar:GetStatusBarTexture() then 
		statBar:SetStatusBarTexture(statbarStyle.texture)
		SetFrameByStyle(statBar, statbarStyle, extended, alpha)
		statBar:SetOrientation(statbarStyle.orientation)
	else statBar:SetAlpha(alpha) end
end
-- >>> ------------------------------------------------------------------------------------------------- >>>
local function ApplyFontStyle(text, style) 																								
	text:SetFont(style.typeface, style.size, "NONE")
	text:SetJustifyH(style.align)
	text:SetJustifyV(style.vertical)
	text:SetDrawLayer("ARTWORK")
	text:SetShadowOffset(1, -1)
	if style.shadow then text:SetShadowColor(0,0,0,1)
	else text:SetShadowColor(0,0,0,0) end
end
-- >>> ------------------------------------------------------------------------------------------------- >>>
local function ApplyTexture(region,texture) 
	if texture == region:GetTexture() then return end
	region:SetTexture(texture) 
	region:SetTexCoord(0,1,0,1) 
end
-- >>> ------------------------------------------------------------------------------------------------- >>>
local function ApplyPlateTheme(plate)
	local index, region, content
	local extended = plate.extended
	local bars = extended.bars
	local regions = extended.regions 	
	local style = plate.extended.style
	local alpha = plate.extended.alpha
		if not InCombatLockdown() then
			plate:SetWidth(style.frame.width)
			plate:SetHeight(style.frame.height)
		end
		-- Alignment Frame
		SetFrameByStyle(extended, style.frame, plate, alpha)	
		-- Main overlay
		ApplyTexture(regions.healthBorder, style.healthborder.texture)
		SetFrameByStyle(regions.healthBorder, style.healthborder, plate, alpha)
		-- Mouseover glow
		ApplyTexture(regions.highlightTexture, style.healthborder.glowtexture)
		regions.highlightTexture:SetAllPoints(regions.healthBorder)
		-- Threat Glow
		ApplyTexture(regions.threatIndicator, style.threatborder.texture)
		SetFrameByStyle(regions.threatIndicator, style.threatborder, plate, alpha)
		-- Cast Bar overlay
		ApplyTexture(regions.castBorder, style.castborder.texture)
		SetFrameByStyle(regions.castBorder, style.castborder, plate, alpha)
		regions.castBorder:SetDrawLayer("ARTWORK")
		-- Cast Unstoppable   (regions.castNostop / style.castnostop)
		if style.options.showNostopSpell then
			ApplyTexture(regions.castNostop, style.castnostop.texture)
			SetFrameByStyle(regions.castNostop, style.castnostop, plate, alpha)
		else
			ApplyTexture(regions.castNostop, style.castborder.texture)
			SetFrameByStyle(regions.castNostop, style.castborder, plate, alpha)
		end
		regions.castNostop:SetDrawLayer("ARTWORK")
		-- Name Text
		if style.options.showName then
			regions.nameText:Show()
			ApplyFontStyle(regions.nameText, style.name)
			SetFrameByStyle(regions.nameText, style.name, extended, alpha)
		else regions.nameText:Hide() end
		-- Level Text
		if style.options.showLevel then
			regions.levelText:Show()
			ApplyFontStyle(regions.levelText, style.level)
			SetFrameByStyle(regions.levelText, style.level, extended, alpha)
		else regions.levelText:Hide() end
		-- Special Text 1
		if style.options.showSpecialText then
			regions.specialText:Show()
			ApplyFontStyle(regions.specialText, style.specialText)
			SetFrameByStyle(regions.specialText, style.specialText, extended, alpha)
		else regions.specialText:Hide() end
		-- Special Text 2
		if style.options.showSpecialText2 then
			regions.specialText2:Show()
			ApplyFontStyle(regions.specialText2, style.specialText2)
			SetFrameByStyle(regions.specialText2, style.specialText2, extended, alpha)
		else regions.specialText2:Hide() end
		-- Special Art
		if style.options.showSpecialArt then
			SetFrameByStyle(regions.specialArt, style.specialArt, extended, alpha)
			regions.specialArt:Show()
		else regions.specialArt:Hide() end
		-- Raid Icon/Lucky Charms..
		regions.raidIcon:SetDrawLayer("OVERLAY")
		SetFrameByStyle(regions.raidIcon, style.raidicon, extended, alpha)
		-- Spellcast Icon
		SetFrameByStyle(regions.spellIcon, style.spellicon, extended, alpha)
		regions.spellIcon:SetDrawLayer("OVERLAY")
		if not style.options.showspellIcon then regions.spellIcon:SetAlpha(0) end
		-- Skull Icon
		if style.options.showDangerSkull then
			regions.dangerSkull:SetDrawLayer("OVERLAY")
			SetFrameByStyle(regions.dangerSkull, style.dangerskull, extended, alpha)
		else regions.dangerSkull:SetAlpha(0) end
		-- Hide redundant items
		regions.threatGlow:SetTexture(style.frame.emptyTexture )
		regions.eliteIcon:SetAlpha(0)
		------------ Configure the Status Bars (Health and Cast)  ------------
		ApplyBarSettings(bars.health, style.healthbar, extended, alpha)
		ApplyBarSettings(bars.cast, style.castbar, extended, alpha)		

end
-- >>> ------------------------------------------------------------------------------------------------- >>>
local function UpdatePlateIndicators(plate)
	local scale
	local extended = plate.extended
	local unit = extended.unit
	local style = extended.style
	local color = extended.color
	local bars = extended.bars
	local regions = extended.regions 
	local threatregion = regions.threatIndicator
	local alpha = extended.alpha
	-- Elite
	if unit.isElite then 
		threatregion:SetTexture(style.threatborder.elitetexture)
		ApplyTexture(regions.healthBorder, style.healthborder.elitetexture)
	else threatregion:SetTexture(style.threatborder.texture) end
	-- Threat
	if shouldDisplayThreat(unit) then
		color = style.threatcolor[unit.threatSituation]
		threatregion:Show()
		threatregion:SetVertexColor(color.r, color.g, color.b, color.a)
		threatregion:SetAlpha(alpha) 
	else threatregion:Hide() end
	-- Special-case Text
	if style.options.showSpecialText and plateTheme.SetSpecialText then
		regions.specialText:SetText(plateTheme.SetSpecialText(unit)) end
	if style.options.showSpecialText2 and plateTheme.SetSpecialText2 then
		regions.specialText2:SetText(plateTheme.SetSpecialText2(unit)) end
	-- Special Case Image
	if style.options.showSpecialArt and plateTheme.SetSpecialArt then
		regions.specialArt:SetTexture(plateTheme.SetSpecialArt(unit)) end
	-- Scale
	if plateTheme.SetScale then
		scale = plateTheme.SetScale(unit)
		extended:SetScale(scale)
		bars.health:SetScale(scale)
		bars.cast:SetScale(scale)
	end

end
-- >>> ------------------------------------------------------------------------------------------------- >>>
local function GetRegionReference(plate, ...)
	local index, region, position
	local select = select
	local regions = plate.extended.regions
	for index = 1, select( "#", ... ) do
		region = select( index, ... )
		position = regionPosition[index]
		if region and position then regions[regionPosition[index]] = region end
	end
end
-- >>> ------------------------------------------------------------------------------------------------- >>>
local function UpdatePlate(statusbar, forceupdate)
	local _, stylename
	local plate = statusbar:GetParent()
	local extended = plate.extended
	local bars = extended.bars
	local regions = extended.regions 
	local unit = plate.extended.unit
	local alpha = 1
	bars.health, bars.cast = plate:GetChildren()
	------------ Derived Properties  ------------
	unit.threatSituation = threatByColor(regions.threatGlow, regions.nameText:GetText())
	unit.reaction, unit.type = reactionByColor(bars.health)
	unit.isDangerous = regions.dangerSkull:IsShown()
	unit.isElite = regions.eliteIcon:IsShown()
	unit.name = regions.nameText:GetText()
	unit.isTarget = (plate:GetAlpha() > .9) and (UnitName("target") == unit.name)	
	unit.level = regions.levelText:GetText()
	unit.health = bars.health:GetValue()
	unit.red, unit.green, unit.blue = bars.health:GetStatusBarColor()
	unit.isCasting = bars.cast:IsShown()
	_, unit.healthmax = bars.health:GetMinMaxValues()
	unit.depth = plate:GetEffectiveDepth();
	--unit.reference -- possible new feature
	------------ Get Current Plate Style ------------
	if plateTheme.multiStyle then 
		stylename = plateTheme.SetStyle(unit)
		plate.extended.style = plateTheme[stylename]
	else plate.extended.style = plateTheme end
	----------- Get Opacity  -------- 
	if plateTheme.SetAlpha then alpha = plateTheme.SetAlpha(unit) end  	
	------------ Configure Graphical Elements ------------
	if forceupdate or extended.stylename ~= stylename or extended.alpha ~= alpha then
		extended.stylename = stylename
		extended.alpha = alpha
		ApplyPlateTheme(plate)
	end
	------------ Indicators  ------------
	UpdatePlateIndicators(plate)
end
-- >>> ------------------------------------------------------------------------------------------------- >>>
local function ApplyPlateExtension(plate)
	local health, cast, extended, regions, fcache
	health, cast = plate:GetChildren()
	plate.extended = CreateFrame("Frame", nil, plate)
	extended = plate.extended
	extended.style = {}
	extended.unit = {}
	extended.color = {}
	extended.regions = {}
	extended.bars = {}
	regions = extended.regions
	GetRegionReference(plate, plate:GetRegions())
	-- Add custom frame elements
	extended.stylename = ""
	extended.alpha = 1
	for index, region in pairs(regions) do region:SetParent(extended) end
	plate.extended:EnableDrawLayer( "HIGHLIGHT" );
	regions.threatIndicator = extended:CreateTexture(nil, "ARTWORK")
	regions.specialArt = extended:CreateTexture(nil, "OVERLAY")
	regions.specialText = extended:CreateFontString(nil, "OVERLAY")
	regions.specialText2 = extended:CreateFontString(nil, "OVERLAY")
	-- Update Immediately
	UpdatePlate(health, true)
	-- Hook for Updates
	if extended.style.options.useOnUpdate then
		health:HookScript("OnShow", function (self)  UpdatePlate(self, true)  end) 
		health:HookScript("OnUpdate", function (self) UpdatePlate(self, false) end) 
	else
		health:HookScript("OnValueChanged", function (self) UpdatePlate(self, false) end) 
		health:HookScript("OnShow", function (self)  UpdatePlate(self, true)  end) 
		health:HookScript("OnEvent", function (self, event) UpdatePlate(self, false)  end)
		health:RegisterEvent("PLAYER_REGEN_DISABLED")
		health:RegisterEvent("PLAYER_REGEN_ENABLED")
		health:RegisterEvent("PLAYER_TARGET_CHANGED")
		health:RegisterEvent("UNIT_THREAT_LIST_UPDATE")
	end
	-- Hook for Castbar
	cast:HookScript("OnShow", function (self) UpdatePlate(self, true) end) 
	cast:HookScript("OnValueChanged", function (self) UpdatePlate(self, true) end) 
	cast:HookScript("OnHide", function (self) UpdatePlate(self, false) end) 
end
-- >>> ------------------------------------------------------------------------------------------------- >>>
local function IsFrameNameplate(frame)
	local region = frame:GetRegions()
	return region and region:GetObjectType() == "Texture" and region:GetTexture() == "Interface\\TargetingFrame\\UI-TargetingFrame-Flash" 
end
-- >>> ------------------------------------------------------------------------------------------------- >>>	
local function OnCreateChildFrames(...)
	for i=1, select("#", ...) do 
		local plate = select(i, ...)
		if IsFrameNameplate(plate) then
			if not plate.extended then ApplyPlateExtension(plate) end
		end end
end
-- >>> ------------------------------------------------------------------------------------------------- >>>
local function OnUpdate(self)
	local curChildren = WorldFrame:GetNumChildren()
	if (curChildren ~= numChildren) then
		numChildren = curChildren
		OnCreateChildFrames(WorldFrame:GetChildren()) 
	end	
end
-- >>> ------------------------------------------------------------------------------------------------- >>>
PlateHandler:SetScript("OnEvent", function(self, event)
	if event == "PLAYER_ENTERING_WORLD" then self:SetScript("OnUpdate", OnUpdate) end
end)
PlateHandler:RegisterEvent("PLAYER_ENTERING_WORLD")
-- >>> ------------------------------------------------------------------------------------------------- >>>
function TidyPlates:UseTheme(theme)
	if theme and type(theme) == 'table' then
		plateTheme = LoadTheme(TidyPlatesThemeTemplate, theme)
	end
end
-- >>> ------------------------------------------------------------------------------------------------- >>>







